/* LED counter GNOME panel applet
 * Insprired by Sixtyfour bit counter by Tim Gerla
 * and xled by Jan Schoenepauck <schoenep@uni-wuppertal.de>
 * and Joachim Gassen <gassen@uni-muenster.de>"
 *
 * By Uwe Steinmann <Uwe.Steinmann@fernuni-hagen.de>
 *
 * Enjoy.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or 
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License  
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <gnome.h>
#include <applet-widget.h>
#include <math.h>
#include "gtkdisplay.h"
#include "ledcount.h"

GtkWidget *app;
DISP_INFO disp_info[MAX_DISPLAYS];
int height, width;
int displays;
int new_displays;
int spacing = 10;
GdkColor on, off, over;
gchar con_s[32], coff_s[32], cover_s[32];

int check_vbox_messages (void);
int count_mail (char *file, int all);
int count_timer(GtkWidget *data);

extern void prefs_cb(GtkWidget *w, gpointer p);

GtkWidget *counter = NULL;
GtkWidget *globalbox = NULL;

static void
about_cb(GtkWidget *w, gpointer d)
{
	gchar *authors[4];
	GtkWidget *about;
	
	authors[0] = "Uwe Steinmann";
	authors[1] = _("Insprired by Sixtyfour bit counter by Tim Gerla");
	authors[2] = _("and xled by Jan Schoenepauck <schoenep@uni-wuppertal.de>");
	authors[3] = _("and Joachim Gassen <gassen@uni-muenster.de>");
	authors[4] = NULL;
	
	about = gnome_about_new (
		"LED Counter",
		VERSION,
		"(C) 1999 Uwe Steinmann",
		(const gchar**)authors,
		_("This is a general purpose counter. It displays the counted number"
      " on an old fashion 7 segment display. It can count mail messages"
      " or simple display the return value of any command, making"
	    " it possible to count almost everything."
		  " Since the return value can not be greater than 255 it's"
		  " not always very useful, but I'm working on it."
		  " The number of displays is limited to 10 with a maximum of 10 digits."),
		NULL);
	
	gtk_widget_show(about);
	
	return;
}

void
change_spacing(int space)
{
	GNOME_Panel_OrientType orient;

	orient = applet_widget_get_panel_orient (APPLET_WIDGET (app));
	switch (orient) {
		case GNOME_Panel_ORIENT_LEFT:
		case GNOME_Panel_ORIENT_RIGHT:
//			gtk_table_set_row_spacings(GTK_TABLE(counter), space);
			gtk_box_set_spacing(GTK_BOX(counter), space);
			break;
		default:
//			gtk_table_set_col_spacings(GTK_TABLE(counter), space);
			gtk_box_set_spacing(GTK_BOX(counter), space);
			break;
	}
}

int
count_timer(GtkWidget *data)
{
	gint z, zstart, zend;
	gint count = 0;

	/**** If data = NULL count for all displays *****/
	if(NULL == data) {
		zstart = 0;
		zend = displays;
	} else {
		zstart = (gint) data-1;
		zend = (gint) data;
	}
 	for(z = zstart; z < zend; z++) {
		if (disp_info[z].mail_mode) {
			count = count_mail(disp_info[z].mailbox_file, disp_info[z].show_all_messages);
		} else if (disp_info[z].front_end_mode) {
			FILE *fp = popen(disp_info[z].backend_command, "r");
			if(NULL == fp) {
				count = 0;
			} else {
				fscanf(fp, "%d", &count);
				pclose(fp);
			}
		} else
			count = check_vbox_messages();

		gtk_display_set_value(GTK_DISPLAY(disp_info[z].display), count);
		if(gtk_display_get_autorange(GTK_DISPLAY(disp_info[z].display)))
			gtk_widget_set_usize((GTK_WIDGET(disp_info[z].display)),
			  gtk_display_get_digits(GTK_DISPLAY(disp_info[z].display))*width,
			  height);
	}

	return 1;
}

gint run_command(GtkWidget *widget, gpointer data) {
  DISP_INFO   *disp_info;

	disp_info = (DISP_INFO *) data;
fprintf(stderr, "disp_info = 0x%X\n", disp_info);
	if(disp_info->exec_command) {
		system(disp_info->exec_command);
  }

	return 0;
}

gint button_pressed_cb(GtkWidget *widget, GdkEventButton *event, gpointer data) {
  DISP_INFO   *disp_info;

	disp_info = (DISP_INFO *) data;
fprintf(stderr, "button %d\n", event->button);
	return 0;
}

void remove_exec_button(DISP_INFO *disp_info)
{
	gtk_widget_destroy(disp_info->label);
}

void add_exec_button(DISP_INFO *disp_info)
{
	disp_info->label = gtk_label_new(disp_info->title);
	gtk_widget_set_usize(disp_info->label, 1, BUTTON_HEIGHT);
	gtk_box_pack_end(GTK_BOX(disp_info->container), disp_info->label, TRUE, TRUE, 1);
	gtk_widget_show_all(disp_info->container);
}

void destroy_counter(GtkWidget *widget)
{
	if(widget == NULL)
		return;
	gtk_widget_destroy(widget);
	counter = NULL;
}

static GtkWidget
*create_counter(GNOME_Panel_OrientType orient)
{
	int x, y, i;
	GtkWidget *table, *button, *hbox;
	
	gdk_color_parse(coff_s, &off);
	gdk_color_parse(con_s, &on);
	gdk_color_parse(cover_s, &over);

	switch (orient) {
		case GNOME_Panel_ORIENT_LEFT:
		case GNOME_Panel_ORIENT_RIGHT:
			table = gtk_vbox_new(FALSE, spacing);
			break;
		default:
			table = gtk_hbox_new(FALSE, spacing);
			break;
	}

	x = y = 0;
	for(i = 0; i < displays; i++)
	{
		GtkWidget *frame, *tip;

		frame = gtk_frame_new(NULL);
		gtk_container_set_border_width(GTK_CONTAINER(frame), 1);
		gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);

		disp_info[i].container = gtk_vbox_new(FALSE, 0);
		gtk_container_add(GTK_CONTAINER(frame), disp_info[i].container);

		disp_info[i].display = gtk_display_new_with_digits(disp_info[i].digits);
		gtk_display_set_autorange(GTK_DISPLAY(disp_info[i].display), disp_info[i].autodigits);
		count_timer((void *) i+1);

		/**** Display ****/
		disp_info[i].button = gtk_button_new();
		GTK_WIDGET_UNSET_FLAGS(disp_info[i].button, GTK_CAN_DEFAULT);
		GTK_WIDGET_UNSET_FLAGS(disp_info[i].button, GTK_CAN_FOCUS);
		tip = (GtkWidget *)gtk_tooltips_new();
		gtk_tooltips_set_tip(GTK_TOOLTIPS(tip), disp_info[i].button, disp_info[i].title, NULL);
		gtk_tooltips_enable(GTK_TOOLTIPS(tip));

	  gtk_signal_connect(GTK_OBJECT(disp_info[i].button), "clicked",
         	  GTK_SIGNAL_FUNC(run_command),
         	  (void *) &(disp_info[i]));
		gtk_container_set_border_width(GTK_CONTAINER(disp_info[i].button), 0);
//		gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
		gtk_container_add(GTK_CONTAINER(disp_info[i].button), disp_info[i].display);
		gtk_display_set_colors(GTK_DISPLAY(disp_info[i].display), &on, &off, &over);
		gtk_widget_set_usize(disp_info[i].display,
		   width*gtk_display_get_digits(GTK_DISPLAY(disp_info[i].display)),
		   height);
		gtk_box_pack_start(GTK_BOX(disp_info[i].container), disp_info[i].button, TRUE, TRUE, 0);
		gtk_signal_connect(GTK_OBJECT(disp_info[i].display), "button_press_event",
		        	GTK_SIGNAL_FUNC(button_pressed_cb),
		        	&disp_info[i]);

		/**** Button ****/
		if(disp_info[i].exec_button) {
			add_exec_button(&disp_info[i]);
		} else {
			gtk_widget_set_usize(disp_info[i].display,
			   width*gtk_display_get_digits(GTK_DISPLAY(disp_info[i].display)),
			   height+BUTTON_HEIGHT);
		}
//		gtk_box_pack_start(GTK_BOX(table), disp_info[i].container, TRUE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(table), frame, TRUE, TRUE, 0);

		switch (orient) {
			case GNOME_Panel_ORIENT_LEFT:
			case GNOME_Panel_ORIENT_RIGHT:
				y++;
				break;
			default:
				x++;
				break;
		}
	}

	gtk_widget_show_all(table);
	return table;
}

static void
applet_change_orient(GtkWidget *w, PanelOrientType o, gpointer data)
{
	int i;

fprintf(stderr, "change orient %s\n", (o == ORIENT_LEFT || o == ORIENT_RIGHT) ? "left right" : "up down");

	gtk_container_remove(GTK_CONTAINER(globalbox), counter);

	switch (o) {
		case ORIENT_LEFT:
		case ORIENT_RIGHT:
			counter = gtk_vbox_new(FALSE, 0);
			break;
		default:
			counter = gtk_hbox_new(FALSE, 0);
			break;
	}

	for(i = 0; i < displays; i++)
	{
		gtk_box_pack_start(GTK_BOX(counter), disp_info[i].container, TRUE, TRUE, 0);
	}
	gtk_container_add(GTK_CONTAINER(globalbox), counter);
}

static void
bye(GtkWidget *w, gpointer p)
{
	int i;
	char buffer[300];

	gnome_config_set_int("ledcount/general/displays", new_displays);
	gnome_config_set_int("ledcount/general/width", width);
	gnome_config_set_int("ledcount/general/height", height);
	gnome_config_set_int("ledcount/general/spacing", spacing);
	sprintf(buffer, "ledcount/general/color_on");
	gnome_config_set_string(buffer, con_s);
	sprintf(buffer, "ledcount/general/color_off");
	gnome_config_set_string(buffer, coff_s);
	sprintf(buffer, "ledcount/general/color_over");
	gnome_config_set_string(buffer, cover_s);
	for(i=0; i<displays; i++) {
		sprintf(buffer, "ledcount/general/mailbox%d", i+1);
		gnome_config_set_string(buffer, disp_info[i].mailbox_file);
		sprintf(buffer, "ledcount/general/execute%d", i+1);
		gnome_config_set_string(buffer, disp_info[i].exec_command);
		sprintf(buffer, "ledcount/general/title%d", i+1);
		gnome_config_set_string(buffer, disp_info[i].title);
		sprintf(buffer, "ledcount/general/backend%d", i+1);
		gnome_config_set_string(buffer, disp_info[i].backend_command);
		sprintf(buffer, "ledcount/general/digits%d", i+1);
		gnome_config_set_int(buffer, disp_info[i].digits);
		sprintf(buffer, "ledcount/general/mailmode%d", i+1);
		gnome_config_set_int(buffer, disp_info[i].mail_mode);
		sprintf(buffer, "ledcount/general/showall%d", i+1);
		gnome_config_set_int(buffer, disp_info[i].show_all_messages);
		sprintf(buffer, "ledcount/general/frontendmode%d", i+1);
		gnome_config_set_int(buffer, disp_info[i].front_end_mode);
		sprintf(buffer, "ledcount/general/autorange%d", i+1);
		gnome_config_set_int(buffer, disp_info[i].autodigits);
		sprintf(buffer, "ledcount/general/button%d", i+1);
		gnome_config_set_int(buffer, disp_info[i].exec_button);
		sprintf(buffer, "ledcount/general/updatetime%d", i+1);
		gnome_config_set_int(buffer, disp_info[i].updatetime);
	}
	gnome_config_sync();
	gnome_config_drop_all();
	applet_widget_gtk_main_quit();
}

static int
save_session(GtkWidget *w, gpointer p)
{
	int i;
	char buffer[300];

	gnome_config_set_int("ledcount/general/displays", new_displays);
	gnome_config_set_int("ledcount/general/width", width);
	gnome_config_set_int("ledcount/general/height", height);
	gnome_config_set_int("ledcount/general/spacing", spacing);
	sprintf(buffer, "ledcount/general/color_on");
	gnome_config_set_string(buffer, con_s);
	sprintf(buffer, "ledcount/general/color_off");
	gnome_config_set_string(buffer, coff_s);
	sprintf(buffer, "ledcount/general/color_over");
	gnome_config_set_string(buffer, cover_s);
	for(i=0; i<displays; i++) {
		sprintf(buffer, "ledcount/general/mailbox%d", i+1);
		gnome_config_set_string(buffer, disp_info[i].mailbox_file);
		sprintf(buffer, "ledcount/general/execute%d", i+1);
		gnome_config_set_string(buffer, disp_info[i].exec_command);
		sprintf(buffer, "ledcount/general/title%d", i+1);
		gnome_config_set_string(buffer, disp_info[i].title);
		sprintf(buffer, "ledcount/general/backend%d", i+1);
		gnome_config_set_string(buffer, disp_info[i].backend_command);
		sprintf(buffer, "ledcount/general/digits%d", i+1);
		gnome_config_set_int(buffer, disp_info[i].digits);
		sprintf(buffer, "ledcount/general/mailmode%d", i+1);
		gnome_config_set_int(buffer, disp_info[i].mail_mode);
		sprintf(buffer, "ledcount/general/showall%d", i+1);
		gnome_config_set_int(buffer, disp_info[i].show_all_messages);
		sprintf(buffer, "ledcount/general/frontendmode%d", i+1);
		gnome_config_set_int(buffer, disp_info[i].front_end_mode);
		sprintf(buffer, "ledcount/general/autorange%d", i+1);
		gnome_config_set_int(buffer, disp_info[i].autodigits);
		sprintf(buffer, "ledcount/general/button%d", i+1);
		gnome_config_set_int(buffer, disp_info[i].exec_button);
		sprintf(buffer, "ledcount/general/updatetime%d", i+1);
		gnome_config_set_int(buffer, disp_info[i].updatetime);
	}
	gnome_config_sync();
	gnome_config_drop_all();
	return FALSE;
}

int
main(int argc, char *argv[])
{
	int i;
	GNOME_Panel_OrientType orient;
	
	/* initialize the i18n stuff */
	textdomain (PACKAGE);
	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
	
	for(i = 0; i < MAX_DISPLAYS; i++)
	{
		disp_info[i].mail_mode = 1;
		disp_info[i].front_end_mode = 0;
		disp_info[i].show_all_messages = 0;
		disp_info[i].mailbox_file = NULL;
		disp_info[i].front_end_mode = 0;
		disp_info[i].exec_command = NULL;
		disp_info[i].title = NULL;
		disp_info[i].digits = 3;
		disp_info[i].autodigits = 1;
		disp_info[i].exec_button = 1;
	}

	applet_widget_init("ledcount_applet", VERSION, argc, argv, NULL, 0, NULL);
	app = applet_widget_new("ledcount_applet");
	displays = gnome_config_get_int("ledcount/general/displays=1");
	if(displays > MAX_DISPLAYS)
		displays = MAX_DISPLAYS;
	new_displays = displays;
	width = gnome_config_get_int("ledcount/general/width=51");
	height = gnome_config_get_int("ledcount/general/height=30");
	spacing = gnome_config_get_int("ledcount/general/spacing=5");
	sprintf(con_s, "%s", gnome_config_get_string("ledcount/general/color_on=#FF1710"));
	sprintf(coff_s, "%s", gnome_config_get_string("ledcount/general/color_off=#400000"));
	sprintf(cover_s, "%s", gnome_config_get_string("ledcount/general/color_over=#17FF00"));

	for(i=0; i<displays; i++) {
// FIXME: maybe we have to use strdup here because apply_cb frees the memory
		char buffer[50];
		sprintf(buffer, "ledcount/general/mailbox%d=", i+1);
		disp_info[i].mailbox_file = gnome_config_get_string(buffer);
		sprintf(buffer, "ledcount/general/execute%d=", i+1);
		disp_info[i].exec_command = gnome_config_get_string(buffer);
		sprintf(buffer, "ledcount/general/title%d=%d", i+1, i+1);
		disp_info[i].title = gnome_config_get_string(buffer);
		sprintf(buffer, "ledcount/general/backend%d=", i+1);
		disp_info[i].backend_command = gnome_config_get_string(buffer);
		sprintf(buffer, "ledcount/general/digits%d=3", i+1);
		disp_info[i].digits = gnome_config_get_int(buffer);
		sprintf(buffer, "ledcount/general/mailmode%d=1", i+1);
		disp_info[i].mail_mode = gnome_config_get_int(buffer);
		sprintf(buffer, "ledcount/general/showall%d=0", i+1);
		disp_info[i].show_all_messages = gnome_config_get_int(buffer);
		sprintf(buffer, "ledcount/general/frontendmode%d=0", i+1);
		disp_info[i].front_end_mode = gnome_config_get_int(buffer);
		sprintf(buffer, "ledcount/general/autorange%d=0", i+1);
		disp_info[i].autodigits = gnome_config_get_int(buffer);
		sprintf(buffer, "ledcount/general/button%d=1", i+1);
		disp_info[i].exec_button = gnome_config_get_int(buffer);
		sprintf(buffer, "ledcount/general/updatetime%d=100000", i+1);
		disp_info[i].updatetime = gnome_config_get_int(buffer);
		disp_info[i].timeout = gtk_timeout_add(disp_info[i].updatetime, (GtkFunction)count_timer, (void *) i+1);
	}

	if(!app)
		g_error("Cannot create ledcount applet!");

	orient = applet_widget_get_panel_orient (APPLET_WIDGET (app));
	globalbox = gtk_vbox_new(FALSE, 0);
	counter = create_counter(orient);
	gtk_container_add(GTK_CONTAINER(globalbox),counter);
	applet_widget_add(APPLET_WIDGET(app), globalbox);
	gtk_widget_show_all(app);

	gtk_signal_connect(GTK_OBJECT(app), "delete_event",
			   GTK_SIGNAL_FUNC(bye), NULL);
	gtk_signal_connect(GTK_OBJECT(app), "save_session",
			   GTK_SIGNAL_FUNC(save_session), NULL);
//	gtk_signal_connect(GTK_OBJECT(app),"change_orient",
//	           GTK_SIGNAL_FUNC(applet_change_orient), NULL);

	applet_widget_register_stock_callback(APPLET_WIDGET(app),
					      "about",                    
					      GNOME_STOCK_MENU_ABOUT,
					      _("About"),         
					      GTK_SIGNAL_FUNC(about_cb),
					      NULL);

	applet_widget_register_stock_callback(APPLET_WIDGET(app),
					      "preference",                    
					      GNOME_STOCK_MENU_PREF,
					      _("Preferences"),         
					      GTK_SIGNAL_FUNC(prefs_cb),
					      NULL);

	count_timer(NULL);
	
	applet_widget_gtk_main();
	return(0);
}
